import os
from typing import Dict, List

from wgtool.models import WGConfigGroup, WGPeerConfig, WGServerConfig


class WGConfigExporter:
    def to_text(self, config: List[WGConfigGroup]) -> str:
        key_width = max(len(key) for group in config for key in group.key_values)
        text = ""
        for group in config:
            text += f"\n[{group.name}]\n"
            for key, value in group.key_values.items():
                text += f"{key: <{key_width}} = {value}\n"
        return text.lstrip("\n")

    def _to_file(self, config: List[WGConfigGroup], file: str, mode: int = 0o600) -> str:
        directory = os.path.dirname(os.path.realpath(file))
        text = self.to_text(config)
        os.makedirs(directory, mode=mode, exist_ok=True)
        with open(file, "w") as f:
            f.write(text)
        os.chmod(file, mode=mode)
        return text


class WGServerConfigExporter(WGConfigExporter):
    def __init__(self, server_config: WGServerConfig) -> None:
        self.config = server_config

    def to_file(self, file: str) -> str:
        interface = self.get_interface_group()
        peers = self.get_peer_groups()
        return self._to_file([interface] + peers, file)

    def get_interface_group(self) -> WGConfigGroup:
        interface: Dict[str, str] = {
            "Address": self.config.address,
            "ListenPort": str(self.config.server.port),
            "MTU": str(self.config.mtu),
            "PostUp": self.config.post_up,
            "PostDown": self.config.post_down,
            "PrivateKey": self.config.server.private_key,
            **self.config.others,
            "# PublicKey": self.config.server.public_key,
            "# Endpoint": self.config.server.endpoint,
        }
        return WGConfigGroup("Interface", interface)

    def get_peer_groups(self) -> List[WGConfigGroup]:
        peers: List[WGConfigGroup] = []
        for peer in self.config.peers:
            peer_attributes = {
                "# Name": peer.name,
                "PublicKey": peer.public_key,
                "PresharedKey": peer.preshared_key,
                "AllowedIPs": peer.address,
                **peer.others,
            }
            peers.append(WGConfigGroup("Peer", peer_attributes))
        return peers


class WGPeerConfigExporter(WGConfigExporter):
    def __init__(self, peer_config: WGPeerConfig) -> None:
        self.peer_config = peer_config

    def to_file(self, file: str) -> str:
        interface = self.get_interface_group()
        peer = self.get_peer_group()
        return self._to_file([interface, peer], file)

    def get_interface_group(self) -> WGConfigGroup:
        interface_attributes = {
            "Address": self.peer_config.address,
            "DNS": ", ".join(str(ip) for ip in self.peer_config.dns),
            "PrivateKey": self.peer_config.private_key,
        }
        interface_attributes = {k: v for k, v in interface_attributes.items() if v}
        return WGConfigGroup("Interface", interface_attributes)

    def get_peer_group(self) -> WGConfigGroup:
        peer_attributes = {
            "Endpoint": self.peer_config.server.endpoint,
            "AllowedIPs": ", ".join(str(ip) for ip in self.peer_config.allowed_ips),
            "PresharedKey": self.peer_config.preshared_key,
            "PublicKey": self.peer_config.server.public_key,
        }
        return WGConfigGroup("Peer", peer_attributes)



import qrcode
from qrcode.main import QRCode


def print_qrcode(text: str) -> None:
    qr = QRCode(
        version=1,
        error_correction=qrcode.constants.ERROR_CORRECT_L,
        box_size=1,
        border=2,  # Add some padding to make the QR code more readable
    )

    # Add data to the QR code
    qr.add_data(text)
    qr.make(fit=True)
    
    # Generate the QR code matrix (1s and 0s)
    qr_matrix = qr.get_matrix()
    
    # Print the QR code two rows at a time to create a more compact output
    for y in range(0, len(qr_matrix), 2):
        line = ''
        for x in range(len(qr_matrix[y])):
            upper_pixel = qr_matrix[y][x]
            lower_pixel = qr_matrix[y + 1][x] if y + 1 < len(qr_matrix) else 0
            
            if upper_pixel and lower_pixel:
                line += ' '  # Empty, show black
            elif upper_pixel and not lower_pixel:
                line += '▄'  # Lower filled, upper not filled
            elif not upper_pixel and lower_pixel:
                line += '▀'  # Upper filled, lower not filled
            else:
                line += '█'  # Both filled, show whit
        print(line)





import os
import subprocess
import tempfile

from wgtool.exceptions import WGToolError


def generate_preshared_key() -> str:
    return _run_command("wg genpsk")


def generate_private_key() -> str:
    return _run_command("wg genkey")


def generate_public_key(private: str) -> str:
    with tempfile.TemporaryDirectory() as tempdir:
        file = os.path.join(tempdir, "key")
        with open(file, "w") as f:
            f.write(private)
        try:
            if os.name == "nt":
                return _run_command(f'type "{file}" | wg pubkey')
            return _run_command(f'cat "{file}" | wg pubkey')
        except subprocess.CalledProcessError as e:
            raise WGToolError(f"Invalid private key: {private}") from e


def _run_command(command: str) -> str:
    try:
        return subprocess.check_output(command, shell=True, encoding="utf-8").strip()
    except Exception as e:
        if "127" in str(e) and "wg " in str(e):
            raise WGToolError('WireGuard command "wg" not found. Is wireguard installed?')
        raise WGToolError(f'Error executing command "{command}": {e}')



import os
import re
import shutil
import subprocess


def default_interface() -> str:
    try:
        command = 'ip route list default | grep -Eo " dev ([0-9a-z]+)"'
        return subprocess.check_output(command, shell=True, encoding="utf-8").strip().split()[1]
    except Exception:
        return "eth0"


def enable_forwarding(ipv4: bool, ipv6: bool) -> None:
    if os.name != "posix":
        return

    file = "/etc/sysctl.conf"
    try:
        with open(file) as f:
            content = f.read()
            new_content = content

        # IPv4
        if ipv4:
            if re.search(r"net.ipv4.ip_forward\s*=\s*1$", new_content, re.MULTILINE):
                new_content = re.sub(
                    r"#\s*net.ipv4.ip_forward\s*=\s*1",
                    "net.ipv4.ip_forward=1",
                    new_content,
                    re.MULTILINE,
                )
            else:
                new_content = new_content.strip("\n") + "\nnet.ipv4.ip_forward=1\n"

        # IPv6
        if ipv6:
            if re.search(r"net.ipv6.conf.all.forwarding\s*=\s*1", new_content, re.MULTILINE):
                new_content = re.sub(
                    r"#\s*net.ipv6.conf.all.forwarding\s*=\s*1",
                    "net.ipv6.conf.all.forwarding=1",
                    new_content,
                    re.MULTILINE,
                )
            else:
                new_content = new_content.strip("\n") + "\nnet.ipv6.conf.all.forwarding=1\n"
        if content == new_content:
            return

        print(f'Updating "{file}"')
        file_backup = f"{file}.wgtool"
        if not os.path.exists(file_backup):
            shutil.copy(file, file_backup)
        with open(file, "w") as f:
            f.write(new_content)
        print("Applying changes to sysctl:")
        result = subprocess.check_output("sudo sysctl -p", shell=True, encoding="utf-8").strip()
        print(result)

    except FileNotFoundError:
        print(f'Cannot open "{file}"')


def restart_systemctl_service() -> bool:
    print("Restarting service...")
    if os.name != "posix":
        return False
    commands = [
        "sudo systemctl enable wg-quick@wg0.service",
        "sudo systemctl daemon-reload",
        "sudo systemctl restart wg-quick@wg0",
    ]
    for command in commands:
        result = subprocess.run(command, shell=True, encoding="utf-8")
        if result.returncode != 0:
            if result.stdout:
                print(result.stdout)
            if result.stderr:
                print(result.stderr)
            return False
    return True



import os
import subprocess
from ipaddress import ip_interface

from wgtool.models import WGServerConfig

POST_UP_LINE_NUMBER = "{iptables} -L INPUT --line-number | head -3 | wc -l"
POST_UP = (  # iptables, ifname, network
    "{iptables} -I FORWARD 1 -i %i -o {ifname} -j ACCEPT; "
    "{iptables} -I FORWARD 1 -i {ifname} -o %i -j ACCEPT; "
    "{iptables} -t nat -I POSTROUTING 1 -s {network} -o {ifname} -j MASQUERADE; "
    "{iptables} -I INPUT {ln_minus_1} -i %i -j ACCEPT > /dev/null; "
    "{iptables} -I INPUT {ln} -p udp -m udp --dport {port} -j ACCEPT > /dev/null;"
)
POST_DN = (
    "{iptables} -D FORWARD   -i %i -o {ifname} -j ACCEPT; "
    "{iptables} -D FORWARD   -i {ifname} -o %i -j ACCEPT; "
    "{iptables} -t nat -D POSTROUTING   -s {network} -o {ifname} -j MASQUERADE; "
    "{iptables} -D INPUT   -i %i -j ACCEPT > /dev/null; "
    "{iptables} -D INPUT   -p udp -m udp --dport {port} -j ACCEPT > /dev/null;"
)


def get_post_up(config: WGServerConfig, ifname: str) -> str:
    if os.name == "posix":
        command = []
        if config.ipv4:
            line_number = subprocess.check_output(
                POST_UP_LINE_NUMBER.format(iptables="iptables"), shell=True
            )
            command.append(
                POST_UP.format(
                    iptables="iptables",
                    ifname=ifname,
                    network=ip_interface(config.ipv4).network.with_prefixlen,
                    port=config.server.port,
                    ln=int(line_number),
                    ln_minus_1=int(line_number) - 1,
                )
            )
        if config.ipv6:
            line_number = subprocess.check_output(
                POST_UP_LINE_NUMBER.format(iptables="ip6tables"), shell=True
            )
            command.append(
                POST_UP.format(
                    iptables="ip6tables",
                    ifname=ifname,
                    network=ip_interface(config.ipv6).network.with_prefixlen,
                    port=config.server.port,
                    ln=int(line_number),
                    ln_minus_1=int(line_number) - 1,
                )
            )
        if command:
            return "; ".join(command)
    return ""


def get_post_down(config: WGServerConfig, ifname: str) -> str:
    if os.name == "posix":
        command = []
        if config.ipv4:
            command.append(
                POST_DN.format(
                    iptables="iptables",
                    ifname=ifname,
                    network=ip_interface(config.ipv4).network.with_prefixlen,
                    port=config.server.port,
                )
            )
        if config.ipv6:
            command.append(
                POST_DN.format(
                    iptables="ip6tables",
                    ifname=ifname,
                    network=ip_interface(config.ipv6).network.with_prefixlen,
                    port=config.server.port,
                )
            )
        if command:
            return "; ".join(command)
    return ""



from dataclasses import dataclass
from ipaddress import (
    IPv4Address,
    IPv4Interface,
    IPv4Network,
    IPv6Address,
    IPv6Interface,
    IPv6Network,
)
from typing import Dict, List, Optional, Union

from pydantic import BaseModel, Field, validator

DEFAULT_PORT = 51820
DEFAULT_FILE = "/etc/wireguard/wg0.conf"


class WGServerEndpoint(BaseModel):
    address: str = "host.example.com"
    port: int = Field(default=DEFAULT_PORT, ge=1024, le=65535)
    public_key: str = ""
    private_key: str = ""

    @property
    def endpoint(self) -> str:
        return f"{self.address}:{self.port}"


class WGPeerConfig(BaseModel):
    name: str = Field(pattern=r"[a-zA-Z][a-zA-Z0-9-_]+")
    public_key: str = ""
    preshared_key: str = ""
    private_key: str = ""
    server: WGServerEndpoint = WGServerEndpoint()
    ipv4: Optional[IPv4Interface] = None
    ipv6: Optional[IPv6Interface] = None
    allowed_ips: List[Union[IPv4Network, IPv6Network]] = Field(default_factory=list)
    dns: List[Union[IPv4Address, IPv6Address]] = Field(default_factory=list)
    others: Dict[str, str] = Field(default_factory=dict)

    @property
    def address(self) -> str:
        return ", ".join([str(ip) for ip in [self.ipv4, self.ipv6] if ip])

    @validator("others")
    def valid_others(cls, value: Dict[str, str]) -> Dict[str, str]:
        return {k: v for k, v in value.items() if k[0].upper() == k[0]}

    class Config:
        validate_assignment = True


class WGServerConfig(BaseModel):
    server: WGServerEndpoint = Field(default_factory=lambda: WGServerEndpoint())
    ipv4: IPv4Interface = IPv4Interface("10.6.0.1/24")
    ipv6: Optional[IPv6Interface] = None
    mtu: int = Field(default=1420, ge=1280, le=65535)
    post_up: str = ""
    post_down: str = ""
    others: Dict[str, str] = Field(default_factory=dict)
    peers: List[WGPeerConfig] = Field(default_factory=list)

    @property
    def address(self) -> str:
        return ", ".join([str(ip) for ip in [self.ipv4, self.ipv6] if ip])

    @validator("others")
    def valid_others(cls, value: Dict[str, str]) -> Dict[str, str]:
        return {k: v for k, v in value.items() if k[0].upper() == k[0]}

    @validator("ipv4")
    def ipv4_max_prefix_len(cls, value: IPv4Interface) -> IPv4Interface:
        if value.network.prefixlen > 30:
            raise ValueError("Server interface ipv4 address mask must be <= 30")
        return value

    @validator("ipv6")
    def ipv6_max_prefix_len(cls, value: Optional[IPv6Interface]) -> Optional[IPv6Interface]:
        if value is not None and value.network.prefixlen <= 126:
            raise ValueError("Server interface ipv6 address mask must be <= 126")
        return value

    class Config:
        validate_assignment = True


@dataclass
class WGConfigGroup:
    name: str
    key_values: Dict[str, str]



#!/usr/bin/env python3
"""
WireGuard Configuration Tool CLI (wgtool)
by rpcsp (pcunha at hotmail.com) - 10/2021
https://github.com/rpcsp/wgtool
"""
import argparse
import logging
import os
import sys

from pydantic_core import ValidationError

from wgtool import host
from wgtool.exceptions import WGToolError
from wgtool.models import DEFAULT_FILE, DEFAULT_PORT
from wgtool.qrcode import print_qrcode
from wgtool.wgtool import WGTool

logger = logging.getLogger()


def parse_args() -> argparse.Namespace:
    """Parse CLI arguments"""

    default_interface = host.default_interface()
    parser = argparse.ArgumentParser(description="WireGuard Configuration Tool")
    subparser = parser.add_subparsers(dest="action", required=True)

    # server
    parser_server = subparser.add_parser("server", help="Setup new server")
    parser_server.add_argument(
        "domain_name",
        metavar="<server-domain-name>",
        help="Server domain name used by peers. E.g. server1.duckdns.org",
    )
    parser_server.add_argument(
        "-4",
        "--ipv4",
        default="",
        metavar="<ip/mask>",
        help="IPv4 prefix with mask",
    )
    parser_server.add_argument(
        "-6",
        "--ipv6",
        default="",
        metavar="<ip/mask>",
        help="IPv6 prefix with mask",
    )
    parser_server.add_argument(
        "-p",
        "--port",
        dest="ListenPort",
        type=int,
        default=DEFAULT_PORT,
        help="UDP port",
    )

    # show
    parser_show = subparser.add_parser("show", help="show peers")
    parser_show.add_argument(
        "name",
        nargs="?",
        help="Peer name or number to be added",
    )

    # add
    parser_add = subparser.add_parser("add", help="Add peer")
    parser_add.add_argument(
        "name",
        help="Name to identify this new peer",
    )
    parser_add.add_argument(
        "-n",
        "--dns",
        nargs="+",
        help="Optional, one or two DNS servers",
    )
    parser_add.add_argument(
        "-s",
        "--split-tunnel",
        action="store_true",
        help="Configure split tunneling (allow LAN access)",
    )

    # delete
    parser_del = subparser.add_parser("delete", help="Delete peer")
    parser_del.add_argument(
        "name",
        help="Peer name or number to be added",
    )

    # others
    parser.add_argument(
        "-f",
        "--file",
        metavar="<conf-file>",
        required=(os.name == "nt"),
        help=(
            f"For Windows users mainly, path to wireguard config file. "
            f"Defaults to {DEFAULT_FILE} for linux"
        ),
    )
    parser.add_argument(
        "-i",
        "--ifname",
        metavar="<interface>",
        default=default_interface,
        help=(
            "For linux users, define LAN interface used with iptables rules. "
            f'Defaults to "{default_interface}"'
        ),
    )
    parser.add_argument(
        "-d",
        "--debug",
        action="store_true",
        help="Enable debug mode",
    )

    args = parser.parse_args()
    if not args.file and os.name != "nt":
        args.file = DEFAULT_FILE
    return args


def action_server(wg: WGTool, config: argparse.Namespace) -> None:
    """Creates server config"""

    if wg.server_config_file_present():
        response = input(
            "Do you want to overwrite the existing configuration and delete any peers? [y/N] "
        )
        if response.lower()[:1] != "y":
            sys.exit(0)

    params = {k: v for k, v in vars(config).items() if k != "action"}
    wg.set_config(**params)
    wg.save_config()
    print(f'Content of the new configuration file "{wg.file}":')
    print(wg.config.model_dump_json(indent=4, exclude_unset=False))
    wg.enable_forwarding()
    wg.restart_systemctl_service()


def action_show(wg: WGTool, args: argparse.Namespace) -> None:
    """Show configured clients"""

    wg.load_config()
    if args.name is None:
        print("Configured peers:")
        if not wg.peers:
            print("  none")
        for index, peer in enumerate(wg.peers):
            print(f"  {index: >2}) {peer.name}")
        return

    request_peer = wg.get_peer(name_or_index=args.name)
    if not request_peer:
        raise WGToolError(f"Peer name or index not found: {args.name}")
    print(request_peer.model_dump_json(indent=4, exclude_defaults=True))


def action_add(wg: WGTool, config: argparse.Namespace) -> None:
    """Adds new client"""

    wg.load_config()
    if wg.get_peer(config.name):
        response = input(f'Peer "{config.name}" exists. Do you want to overwrite it? [y/N] ')
        if response.lower()[:1] != "y":
            sys.exit(0)

    file = wg.add_peer(config.name, config.dns, config.split_tunnel)
    with open(file) as f:
        content = f.read()
    print(f'Peer "{config.name}" config file: {file}\n\n{content}\n')
    print_qrcode(content)
    wg.restart_systemctl_service()


def action_delete(wg: WGTool, config: argparse.Namespace) -> None:
    """Deletes a client"""

    wg.load_config()
    if not wg.get_peer(config.name):
        sys.exit(f'error: Peer "{config.name}" not found')
    wg.peer_delete(config.name)
    wg.restart_systemctl_service()


def main() -> None:
    args = parse_args()
    if args.debug:
        logging.basicConfig(level=logging.DEBUG, format="[%(levelname)s] %(message)s")
    logger.debug(f"CLI arguments: {args}")

    try:
        wg = WGTool(args.file, ifname=args.ifname)
        if args.action in ["add", "delete"]:
            while not args.name:
                args.name = input("Peer name: ")
        if args.action == "server":
            action_server(wg, args)
        elif args.action == "show":
            action_show(wg, args)
        elif args.action == "add":
            action_add(wg, args)
        elif args.action == "delete":
            action_delete(wg, args)

    except PermissionError:
        sys.exit("error: Permission denied. You need root privilege to run this commad.")
    except ValidationError as e:
        for item in e.errors():
            fields = ", ".join([str(e) for e in item["loc"]])
            print(f"failed to validate '{fields}': {item['msg']}")
        sys.exit("error: one or more validations failed")
    except (WGToolError, ValueError) as e:
        sys.exit(f"error: {e}")


if __name__ == "__main__":
    main()



import setuptools


with open("README.md") as f:
    long_description = f.read()

setuptools.setup(
    name="wgtool",
    version="1.1.11",
    author="rpcsp",
    author_email="pcunha@hotmail.com",
    description="WireGuard Configuration Tool",
    license="https://github.com/rpcsp/wgtool/blob/main/LICENSE",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/rpcsp/wgtool",
    project_urls={
        "Project page": "https://github.com/rpcsp/wgtool",
    },
    classifiers=[
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
    ],
    package_dir={"": "src"},
    packages=setuptools.find_packages(where="src"),
    python_requires=">=3.8",
    install_requires=["pydantic", "qrcode"],
    entry_points="""\
    [console_scripts]
    wgtool = wgtool.cli:main
    """,
)



#!/usr/bin/env python3
"""
WireGuard Configuration Tool (wgtool)
by rpcsp (pcunha at hotmail.com) - 10/2021
https://github.com/rpcsp/wgtool
"""
import logging
import os
import pprint
import sys
from ipaddress import IPv4Interface, IPv6Interface, ip_address, ip_interface, ip_network
from typing import List, Literal, Optional, Union, overload

from wgtool import host, post, wgcli
from wgtool.exceptions import WGToolError
from wgtool.exporter import WGPeerConfigExporter, WGServerConfigExporter
from wgtool.importer import WGServerConfigImporter
from wgtool.models import DEFAULT_FILE, WGPeerConfig, WGServerConfig

MIN_PYTHON = (3, 8)
if sys.version_info < MIN_PYTHON:
    sys.exit("Python %s.%s or later is required.\n" % MIN_PYTHON)


DEFAULT_DNS_LIST_IPV4 = ["8.8.8.8", "1.1.1.1"]
DEFAULT_DNS_LIST_IPV6 = ["2001:4860:4860::8888", "2606:4700:4700::1111"]


logger = logging.getLogger()
pp = pprint.PrettyPrinter(indent=4, width=100)


class WGTool:
    def __init__(self, file: str = DEFAULT_FILE, ifname: str = "") -> None:
        self.file = file or DEFAULT_FILE
        self.ifname = ifname or host.default_interface()
        self.config = WGServerConfig()

    def set_config(self, ipv4: str = "", ipv6: str = "", **params: str) -> None:
        if ipv4 or ipv6:
            params["Address"] = f"{ipv4}, {ipv6}".strip(", ")
        WGServerConfigImporter(self.config).read_interface(params)
        self._generate_server_credentials()
        self._generate_post_commands()

    def load_config(self) -> None:
        self.config = WGServerConfigImporter().from_file(self.file)
        self._generate_server_credentials()
        self._generate_post_commands()

    def save_config(self) -> None:
        WGServerConfigExporter(self.config).to_file(self.file)

    def server_config_file_present(self) -> bool:
        return os.path.isfile(self.file)

    def _generate_server_credentials(self) -> None:
        self.config.server.private_key = (
            self.config.server.private_key or wgcli.generate_preshared_key()
        )
        self.config.server.public_key = self.config.server.public_key or wgcli.generate_public_key(
            self.config.server.private_key
        )

    def _generate_post_commands(self) -> None:
        if not self.config.post_down:
            self.config.post_down = post.get_post_down(self.config, self.ifname)
        if not self.config.post_up:
            self.config.post_up = post.get_post_up(self.config, self.ifname)

    def enable_forwarding(self) -> None:
        host.enable_forwarding(ipv4=bool(self.config.ipv4), ipv6=bool(self.config.ipv6))

    def restart_systemctl_service(self) -> None:
        host.restart_systemctl_service()

    @property
    def peers(self) -> List[WGPeerConfig]:
        return self.config.peers

    def get_peer(self, name_or_index: Union[str, int]) -> Optional[WGPeerConfig]:
        for index, peer in enumerate(self.peers):
            if name_or_index in [peer.name, str(index)]:
                return peer
        return None

    def add_peer(
        self, name: str, dns: Optional[List[str]] = None, split_tunnel: bool = False
    ) -> str:
        # Remove peer with same name
        if peer := self.get_peer(name):
            self.config.peers.remove(peer)

        # Keys
        peer_preshared_key = wgcli.generate_preshared_key()
        peer_private_key = wgcli.generate_private_key()
        peer_public_key = wgcli.generate_public_key(peer_private_key)

        ipv4 = self.get_next_address("ipv4", required=True)
        ipv6 = self.get_next_address("ipv6")

        if split_tunnel:
            allowed_ips = ["0.0.0.0/1", "128.0.0.0/1"]
            if ipv6:
                allowed_ips += ["::/1", "8000::/1"]
        else:
            allowed_ips = ["0.0.0.0/0"]
            if ipv6:
                allowed_ips += ["::/0"]
        if not dns:
            dns = [*DEFAULT_DNS_LIST_IPV4]
            if ipv6:
                dns += DEFAULT_DNS_LIST_IPV6

        peer = WGPeerConfig(
            name=name,
            preshared_key=peer_preshared_key,
            private_key=peer_private_key,
            public_key=peer_public_key,
            server=self.config.server,
            ipv4=ipv4,
            ipv6=ipv6,
            allowed_ips=[ip_network(ip) for ip in allowed_ips],
            dns=[ip_address(ip) for ip in dns],
        )
        self.config.peers.append(peer)
        self.save_config()

        file = f"{name}.conf"
        WGPeerConfigExporter(peer).to_file(file)

        logger.debug(f'New peer "{name}"')
        logger.debug(f"  Peer:\n{pp.pformat(peer)}")
        return file

    @overload
    def get_next_address(
        self, ipv: Literal["ipv4"], required: bool = False
    ) -> Optional[IPv4Interface]: ...

    @overload
    def get_next_address(
        self, ipv: Literal["ipv6"], required: bool = False
    ) -> Optional[IPv6Interface]: ...

    def get_next_address(
        self, ipv: Literal["ipv4", "ipv6"], required: bool = False
    ) -> Union[IPv4Interface, IPv6Interface, None]:
        """Return the next available IP address"""
        server_ip: Union[IPv4Interface, IPv6Interface] = getattr(self.config, ipv)
        if not server_ip and not required:
            return None
        if not server_ip:
            raise WGToolError(f"Server {ipv} not found")

        addresses_in_use = [server_ip.ip] + [ip_interface(getattr(p, ipv)).ip for p in self.peers]

        for address in server_ip.network.hosts():
            if address in addresses_in_use:
                continue
            return ip_interface((address, server_ip.network.prefixlen))
        raise WGToolError("Cannot find free IP address for peer")

    def peer_delete(self, name_or_index: str) -> None:
        peer = self.get_peer(name_or_index)
        if not peer:
            raise WGToolError(f"Peer name or index not found: {name_or_index}")
        logger.debug(f'Deleting peer "{peer.name}"')
        self.config.peers.remove(peer)
        self.save_config()



import re
from ipaddress import (
    IPv4Address,
    IPv4Interface,
    IPv6Address,
    IPv6Interface,
    ip_address,
    ip_interface,
)
from typing import Dict, List, Optional, Union

from wgtool.exceptions import WGToolError
from wgtool.models import WGConfigGroup, WGPeerConfig, WGServerConfig

re_group = re.compile(r"\[(\w+)\]")
re_key_value = re.compile(r"^([^=]+)\s*=\s*(.+)$")


class WGServerConfigImporter:
    def __init__(self, current_config: Optional[WGServerConfig] = None) -> None:
        self.config = current_config or WGServerConfig()
        self.interface = WGPeerConfig(name="Interface")
        self.peers: List[WGPeerConfig] = []

    def read_interface(self, attrs: Dict[str, str]) -> None:
        cfg = self.config
        for ip in self._to_ip_interface_list(attrs.pop("Address", "")):
            if isinstance(ip, IPv6Interface):
                self.ipv6 = ip
            else:
                self.ipv4 = ip
        cfg.mtu = int(attrs.pop("MTU", "") or cfg.mtu)
        cfg.post_up = attrs.pop("PostUp", "") or cfg.post_up
        cfg.post_down = attrs.pop("PostDown", "") or cfg.post_down
        cfg.server.private_key = attrs.pop("PrivateKey", "") or cfg.server.private_key
        cfg.server.port = int(attrs.pop("ListenPort", "") or cfg.server.port)
        cfg.server.address = (attrs.pop("# Endpoint", "") or cfg.server.address).split(":")[0]
        cfg.others = {k: v for k, v in attrs.items() if k[0] == k[0].upper()}

    def read_peers(self, peers: List[WGConfigGroup]) -> None:

        class PeerNames:
            def __init__(self) -> None:
                self.i = 0

            def get_name(self) -> str:
                self.i += 1
                return f"Peer {self.i}"

        peer_names = PeerNames()
        for peer in peers:
            name = peer.key_values.pop("# Name", "") or peer_names.get_name()
            pc = WGPeerConfig(name=name)
            for ip in self._to_ip_interface_list(peer.key_values.pop("AllowedIPs", "")):
                if isinstance(ip, IPv4Interface):
                    pc.ipv4 = ip
                else:
                    pc.ipv6 = ip
            pc.public_key = peer.key_values.pop("PublicKey", pc.public_key)
            pc.preshared_key = peer.key_values.pop("PresharedKey", pc.preshared_key)
            pc.dns = self._to_ip_address_list(peer.key_values.pop("DNS", "")) or pc.dns
            pc.server = self.config.server
            pc.others = {k: v for k, v in peer.key_values.items() if k[0] == k[0].upper()}
            self.config.peers.append(pc)

    def from_file(self, file: str) -> WGServerConfig:
        groups = self._from_file(file)
        for i, group in enumerate(groups):
            if group.name == "Interface":
                self.read_interface(group.key_values)
                groups.pop(i)
                break
        else:
            raise WGToolError("Config not found")
        self.read_peers(groups)
        return self.config

    def _from_file(self, file: str) -> List[WGConfigGroup]:
        with open(file) as f:
            content = f.read()

        config: List[WGConfigGroup] = []
        key_values: Dict[str, str] = {}
        for line in content.splitlines():
            line = line.strip()
            if not line:
                continue

            if match := re_group.match(line):
                name = match.group(1).strip()
                key_values = {}
                config.append(WGConfigGroup(name, key_values))
            elif match := re_key_value.match(line):
                key = match.group(1).strip()
                value = match.group(2).strip()
                key_values[key] = value
        return config

    def _to_ip_interface_list(self, address: str) -> List[Union[IPv4Interface, IPv6Interface]]:
        return [ip_interface(ip.strip()) for ip in address.split(",") if ip.strip()]

    def _to_ip_address_list(self, address: str) -> List[Union[IPv4Address, IPv6Address]]:
        return [ip_address(ip.strip()) for ip in address.split(",") if ip.strip()]
